@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2015 by the Quassel Project                        *
+ *   Copyright (C) 2005-2022 by the Quassel Project                        *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,6 +20,9 @@
 
 #include "client.h"
 
+#include <cstdio>
+#include <cstdlib>
+
 #include "abstractmessageprocessor.h"
 #include "abstractui.h"
 #include "bufferinfo.h"
@@ -29,15 +32,17 @@
 #include "bufferviewconfig.h"
 #include "bufferviewoverlay.h"
 #include "clientaliasmanager.h"
+#include "clientauthhandler.h"
 #include "clientbacklogmanager.h"
 #include "clientbufferviewmanager.h"
-#include "clientirclisthelper.h"
 #include "clientidentity.h"
 #include "clientignorelistmanager.h"
+#include "clientirclisthelper.h"
 #include "clienttransfermanager.h"
 #include "clientuserinputhandler.h"
 #include "coreaccountmodel.h"
 #include "coreconnection.h"
+#include "dccconfig.h"
 #include "ircchannel.h"
 #include "ircuser.h"
 #include "message.h"
@@ -47,156 +52,112 @@
 #include "networkmodel.h"
 #include "quassel.h"
 #include "signalproxy.h"
+#include "transfermodel.h"
 #include "util.h"
-#include "clientauthhandler.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-
-QPointer<Client> Client::instanceptr = 0;
-Quassel::Features Client::_coreFeatures = 0;
-
-/*** Initialization/destruction ***/
-
-bool Client::instanceExists()
+Client::Client(std::unique_ptr<AbstractUi> ui, QObject* parent)
+    : QObject(parent)
+    , Singleton<Client>(this)
+    , _signalProxy(new SignalProxy(SignalProxy::Client, this))
+    , _mainUi(std::move(ui))
+    , _networkModel(new NetworkModel(this))
+    , _bufferModel(new BufferModel(_networkModel))
+    , _backlogManager(new ClientBacklogManager(this))
+    , _bufferViewOverlay(new BufferViewOverlay(this))
+    , _coreInfo(new CoreInfo(this))
+    , _ircListHelper(new ClientIrcListHelper(this))
+    , _inputHandler(new ClientUserInputHandler(this))
+    , _transferModel(new TransferModel(this))
+    , _messageModel(_mainUi->createMessageModel(this))
+    , _messageProcessor(_mainUi->createMessageProcessor(this))
+    , _coreAccountModel(new CoreAccountModel(this))
+    , _coreConnection(new CoreConnection(this))
 {
-    return instanceptr;
+#ifdef EMBED_DATA
+    Q_INIT_RESOURCE(data);
+#endif
+
+    connect(mainUi(), &AbstractUi::disconnectFromCore, this, &Client::disconnectFromCore);
+    connect(this, &Client::connected, mainUi(), &AbstractUi::connectedToCore);
+    connect(this, &Client::disconnected, mainUi(), &AbstractUi::disconnectedFromCore);
+
+    connect(this, &Client::networkRemoved, _networkModel, &NetworkModel::networkRemoved);
+    connect(this, &Client::networkRemoved, _messageProcessor, &AbstractMessageProcessor::networkRemoved);
+
+    connect(backlogManager(), &ClientBacklogManager::messagesReceived, _messageModel, &MessageModel::messagesReceived);
+    connect(coreConnection(), &CoreConnection::stateChanged, this, &Client::connectionStateChanged);
+
+    SignalProxy* p = signalProxy();
+
+    p->attachSlot(SIGNAL(displayMsg(Message)), this, &Client::recvMessage);
+    p->attachSlot(SIGNAL(displayStatusMsg(QString,QString)), this, &Client::recvStatusMsg);
+
+    p->attachSlot(SIGNAL(bufferInfoUpdated(BufferInfo)), _networkModel, &NetworkModel::bufferUpdated);
+    p->attachSignal(inputHandler(), &ClientUserInputHandler::sendInput);
+    p->attachSignal(this, &Client::requestNetworkStates);
+
+    p->attachSignal(this, &Client::requestCreateIdentity, SIGNAL(createIdentity(Identity,QVariantMap)));
+    p->attachSignal(this, &Client::requestRemoveIdentity, SIGNAL(removeIdentity(IdentityId)));
+    p->attachSlot(SIGNAL(identityCreated(Identity)), this, &Client::coreIdentityCreated);
+    p->attachSlot(SIGNAL(identityRemoved(IdentityId)), this, &Client::coreIdentityRemoved);
+
+    p->attachSignal(this, &Client::requestCreateNetwork, SIGNAL(createNetwork(NetworkInfo,QStringList)));
+    p->attachSignal(this, &Client::requestRemoveNetwork, SIGNAL(removeNetwork(NetworkId)));
+    p->attachSlot(SIGNAL(networkCreated(NetworkId)), this, &Client::coreNetworkCreated);
+    p->attachSlot(SIGNAL(networkRemoved(NetworkId)), this, &Client::coreNetworkRemoved);
+
+    p->attachSignal(this, &Client::requestPasswordChange, SIGNAL(changePassword(PeerPtr,QString,QString,QString)));
+    p->attachSlot(SIGNAL(passwordChanged(PeerPtr,bool)), this, &Client::corePasswordChanged);
+
+    p->attachSignal(this, &Client::requestKickClient, SIGNAL(kickClient(int)));
+    p->attachSlot(SIGNAL(disconnectFromCore()), this, &Client::disconnectFromCore);
+
+    p->synchronize(backlogManager());
+    p->synchronize(coreInfo());
+    p->synchronize(_ircListHelper);
+
+    coreAccountModel()->load();
+    coreConnection()->init();
 }
 
-
-Client *Client::instance()
-{
-    if (!instanceptr)
-        instanceptr = new Client();
-    return instanceptr;
-}
-
-
-void Client::destroy()
-{
-    if (instanceptr) {
-        delete instanceptr->mainUi();
-        instanceptr->deleteLater();
-        instanceptr = 0;
-    }
-}
-
-
-void Client::init(AbstractUi *ui)
-{
-    instance()->_mainUi = ui;
-    instance()->init();
-}
-
-
-Client::Client(QObject *parent)
-    : QObject(parent),
-    _signalProxy(new SignalProxy(SignalProxy::Client, this)),
-    _mainUi(0),
-    _networkModel(0),
-    _bufferModel(0),
-    _bufferSyncer(0),
-    _aliasManager(0),
-    _backlogManager(new ClientBacklogManager(this)),
-    _bufferViewManager(0),
-    _bufferViewOverlay(new BufferViewOverlay(this)),
-    _ircListHelper(new ClientIrcListHelper(this)),
-    _inputHandler(0),
-    _networkConfig(0),
-    _ignoreListManager(0),
-    _transferManager(0),
-    _messageModel(0),
-    _messageProcessor(0),
-    _coreAccountModel(new CoreAccountModel(this)),
-    _coreConnection(new CoreConnection(this)),
-    _connected(false),
-    _debugLog(&_debugLogBuffer)
-{
-    _signalProxy->synchronize(_ircListHelper);
-}
-
-
 Client::~Client()
 {
     disconnectFromCore();
 }
 
-
-void Client::init()
+AbstractUi* Client::mainUi()
 {
-    _networkModel = new NetworkModel(this);
-
-    connect(this, SIGNAL(networkRemoved(NetworkId)),
-        _networkModel, SLOT(networkRemoved(NetworkId)));
-
-    _bufferModel = new BufferModel(_networkModel);
-    _messageModel = mainUi()->createMessageModel(this);
-    _messageProcessor = mainUi()->createMessageProcessor(this);
-    _inputHandler = new ClientUserInputHandler(this);
-
-    SignalProxy *p = signalProxy();
-
-    p->attachSlot(SIGNAL(displayMsg(const Message &)), this, SLOT(recvMessage(const Message &)));
-    p->attachSlot(SIGNAL(displayStatusMsg(QString, QString)), this, SLOT(recvStatusMsg(QString, QString)));
-
-    p->attachSlot(SIGNAL(bufferInfoUpdated(BufferInfo)), _networkModel, SLOT(bufferUpdated(BufferInfo)));
-    p->attachSignal(inputHandler(), SIGNAL(sendInput(BufferInfo, QString)));
-    p->attachSignal(this, SIGNAL(requestNetworkStates()));
-
-    p->attachSignal(this, SIGNAL(requestCreateIdentity(const Identity &, const QVariantMap &)), SIGNAL(createIdentity(const Identity &, const QVariantMap &)));
-    p->attachSignal(this, SIGNAL(requestRemoveIdentity(IdentityId)), SIGNAL(removeIdentity(IdentityId)));
-    p->attachSlot(SIGNAL(identityCreated(const Identity &)), this, SLOT(coreIdentityCreated(const Identity &)));
-    p->attachSlot(SIGNAL(identityRemoved(IdentityId)), this, SLOT(coreIdentityRemoved(IdentityId)));
-
-    p->attachSignal(this, SIGNAL(requestCreateNetwork(const NetworkInfo &, const QStringList &)), SIGNAL(createNetwork(const NetworkInfo &, const QStringList &)));
-    p->attachSignal(this, SIGNAL(requestRemoveNetwork(NetworkId)), SIGNAL(removeNetwork(NetworkId)));
-    p->attachSlot(SIGNAL(networkCreated(NetworkId)), this, SLOT(coreNetworkCreated(NetworkId)));
-    p->attachSlot(SIGNAL(networkRemoved(NetworkId)), this, SLOT(coreNetworkRemoved(NetworkId)));
-
-    p->attachSignal(this, SIGNAL(requestPasswordChange(PeerPtr,QString,QString,QString)), SIGNAL(changePassword(PeerPtr,QString,QString,QString)));
-    p->attachSlot(SIGNAL(passwordChanged(PeerPtr,bool)), this, SLOT(corePasswordChanged(PeerPtr,bool)));
-
-    //connect(mainUi(), SIGNAL(connectToCore(const QVariantMap &)), this, SLOT(connectToCore(const QVariantMap &)));
-    connect(mainUi(), SIGNAL(disconnectFromCore()), this, SLOT(disconnectFromCore()));
-    connect(this, SIGNAL(connected()), mainUi(), SLOT(connectedToCore()));
-    connect(this, SIGNAL(disconnected()), mainUi(), SLOT(disconnectedFromCore()));
-
-    // attach backlog manager
-    p->synchronize(backlogManager());
-    connect(backlogManager(), SIGNAL(messagesReceived(BufferId, int)), _messageModel, SLOT(messagesReceived(BufferId, int)));
-
-    coreAccountModel()->load();
-
-    connect(coreConnection(), SIGNAL(stateChanged(CoreConnection::ConnectionState)), SLOT(connectionStateChanged(CoreConnection::ConnectionState)));
-    coreConnection()->init();
+    return instance()->_mainUi.get();
 }
 
-
-/*** public static methods ***/
-
-AbstractUi *Client::mainUi()
+bool Client::isCoreFeatureEnabled(Quassel::Feature feature)
 {
-    return instance()->_mainUi;
+    return coreConnection()->peer() ? coreConnection()->peer()->hasFeature(feature) : false;
 }
 
-
-void Client::setCoreFeatures(Quassel::Features features)
-{
-    _coreFeatures = features;
-}
-
-
 bool Client::isConnected()
 {
     return instance()->_connected;
 }
 
-
 bool Client::internalCore()
 {
     return currentCoreAccount().isInternal();
 }
 
+void Client::onDbUpgradeInProgress(bool inProgress)
+{
+    emit dbUpgradeInProgress(inProgress);
+}
+
+void Client::onExitRequested(int exitCode, const QString& reason)
+{
+    if (!reason.isEmpty()) {
+        qCritical() << reason;
+        emit exitRequested(reason);
+    }
+    QCoreApplication::exit(exitCode);
+}
 
 /*** Network handling ***/
 
@@ -205,29 +166,27 @@ QList<NetworkId> Client::networkIds()
     return instance()->_networks.keys();
 }
 
-
-const Network *Client::network(NetworkId networkid)
+const Network* Client::network(NetworkId networkid)
 {
-    if (instance()->_networks.contains(networkid)) return instance()->_networks[networkid];
-    else return 0;
+    if (instance()->_networks.contains(networkid))
+        return instance()->_networks[networkid];
+    else
+        return nullptr;
 }
 
-
-void Client::createNetwork(const NetworkInfo &info, const QStringList &persistentChannels)
+void Client::createNetwork(const NetworkInfo& info, const QStringList& persistentChannels)
 {
     emit instance()->requestCreateNetwork(info, persistentChannels);
 }
 
-
 void Client::removeNetwork(NetworkId id)
 {
     emit instance()->requestRemoveNetwork(id);
 }
 
-
-void Client::updateNetwork(const NetworkInfo &info)
+void Client::updateNetwork(const NetworkInfo& info)
 {
-    Network *netptr = instance()->_networks.value(info.networkId, 0);
+    Network* netptr = instance()->_networks.value(info.networkId, 0);
     if (!netptr) {
         qWarning() << "Update for unknown network requested:" << info;
         return;
@@ -235,39 +194,35 @@ void Client::updateNetwork(const NetworkInfo &info)
     netptr->requestSetNetworkInfo(info);
 }
 
-
-void Client::addNetwork(Network *net)
+void Client::addNetwork(Network* net)
 {
     net->setProxy(signalProxy());
     signalProxy()->synchronize(net);
     networkModel()->attachNetwork(net);
-    connect(net, SIGNAL(destroyed()), instance(), SLOT(networkDestroyed()));
+    connect(net, &QObject::destroyed, instance(), &Client::networkDestroyed);
     instance()->_networks[net->networkId()] = net;
     emit instance()->networkCreated(net->networkId());
 }
 
-
 void Client::coreNetworkCreated(NetworkId id)
 {
     if (_networks.contains(id)) {
         qWarning() << "Creation of already existing network requested!";
         return;
     }
-    Network *net = new Network(id, this);
+    auto* net = new Network(id, this);
     addNetwork(net);
 }
 
-
 void Client::coreNetworkRemoved(NetworkId id)
 {
     if (!_networks.contains(id))
         return;
-    Network *net = _networks.take(id);
+    Network* net = _networks.take(id);
     emit networkRemoved(net->networkId());
     net->deleteLater();
 }
 
-
 /*** Identity handling ***/
 
 QList<IdentityId> Client::identityIds()
@@ -275,28 +230,25 @@ QList<IdentityId> Client::identityIds()
     return instance()->_identities.keys();
 }
 
-
-const Identity *Client::identity(IdentityId id)
+const Identity* Client::identity(IdentityId id)
 {
-    if (instance()->_identities.contains(id)) return instance()->_identities[id];
-    else return 0;
+    if (instance()->_identities.contains(id))
+        return instance()->_identities[id];
+    else
+        return nullptr;
 }
 
-
-void Client::createIdentity(const CertIdentity &id)
+void Client::createIdentity(const CertIdentity& id)
 {
     QVariantMap additional;
-#ifdef HAVE_SSL
     additional["KeyPem"] = id.sslKey().toPem();
     additional["CertPem"] = id.sslCert().toPem();
-#endif
     emit instance()->requestCreateIdentity(id, additional);
 }
 
-
-void Client::updateIdentity(IdentityId id, const QVariantMap &ser)
+void Client::updateIdentity(IdentityId id, const QVariantMap& ser)
 {
-    Identity *idptr = instance()->_identities.value(id, 0);
+    Identity* idptr = instance()->_identities.value(id, 0);
     if (!idptr) {
         qWarning() << "Update for unknown identity requested:" << id;
         return;
@@ -304,17 +256,15 @@ void Client::updateIdentity(IdentityId id, const QVariantMap &ser)
     idptr->requestUpdate(ser);
 }
 
-
 void Client::removeIdentity(IdentityId id)
 {
     emit instance()->requestRemoveIdentity(id);
 }
 
-
-void Client::coreIdentityCreated(const Identity &other)
+void Client::coreIdentityCreated(const Identity& other)
 {
     if (!_identities.contains(other.id())) {
-        Identity *identity = new Identity(other, this);
+        auto* identity = new Identity(other, this);
         _identities[other.id()] = identity;
         identity->setInitialized();
         signalProxy()->synchronize(identity);
@@ -325,20 +275,18 @@ void Client::coreIdentityCreated(const Identity &other)
     }
 }
 
-
 void Client::coreIdentityRemoved(IdentityId id)
 {
     if (_identities.contains(id)) {
         emit identityRemoved(id);
-        Identity *i = _identities.take(id);
+        Identity* i = _identities.take(id);
         i->deleteLater();
     }
 }
 
-
 /*** User input handling ***/
 
-void Client::userInput(const BufferInfo &bufferInfo, const QString &message)
+void Client::userInput(const BufferInfo& bufferInfo, const QString& message)
 {
     // we need to make sure that AliasManager is ready before processing input
     if (aliasManager() && aliasManager()->isInitialized())
@@ -347,7 +295,6 @@ void Client::userInput(const BufferInfo &bufferInfo, const QString &message)
         instance()->_userInputBuffer.append(qMakePair(bufferInfo, message));
 }
 
-
 void Client::sendBufferedUserInput()
 {
     for (int i = 0; i < _userInputBuffer.count(); i++)
@@ -356,7 +303,6 @@ void Client::sendBufferedUserInput()
     _userInputBuffer.clear();
 }
 
-
 /*** core connection stuff ***/
 
 void Client::connectionStateChanged(CoreConnection::ConnectionState state)
@@ -373,33 +319,34 @@ void Client::connectionStateChanged(CoreConnection::ConnectionState state)
     }
 }
 
-
 void Client::setSyncedToCore()
 {
     // create buffersyncer
     Q_ASSERT(!_bufferSyncer);
     _bufferSyncer = new BufferSyncer(this);
-    connect(bufferSyncer(), SIGNAL(lastSeenMsgSet(BufferId, MsgId)), _networkModel, SLOT(setLastSeenMsgId(BufferId, MsgId)));
-    connect(bufferSyncer(), SIGNAL(markerLineSet(BufferId, MsgId)), _networkModel, SLOT(setMarkerLineMsgId(BufferId, MsgId)));
-    connect(bufferSyncer(), SIGNAL(bufferRemoved(BufferId)), this, SLOT(bufferRemoved(BufferId)));
-    connect(bufferSyncer(), SIGNAL(bufferRenamed(BufferId, QString)), this, SLOT(bufferRenamed(BufferId, QString)));
-    connect(bufferSyncer(), SIGNAL(buffersPermanentlyMerged(BufferId, BufferId)), this, SLOT(buffersPermanentlyMerged(BufferId, BufferId)));
-    connect(bufferSyncer(), SIGNAL(buffersPermanentlyMerged(BufferId, BufferId)), _messageModel, SLOT(buffersPermanentlyMerged(BufferId, BufferId)));
-    connect(bufferSyncer(), SIGNAL(bufferMarkedAsRead(BufferId)), SIGNAL(bufferMarkedAsRead(BufferId)));
-    connect(networkModel(), SIGNAL(requestSetLastSeenMsg(BufferId, MsgId)), bufferSyncer(), SLOT(requestSetLastSeenMsg(BufferId, const MsgId &)));
+    connect(bufferSyncer(), &BufferSyncer::lastSeenMsgSet, _networkModel, &NetworkModel::setLastSeenMsgId);
+    connect(bufferSyncer(), &BufferSyncer::markerLineSet, _networkModel, &NetworkModel::setMarkerLineMsgId);
+    connect(bufferSyncer(), &BufferSyncer::bufferRemoved, this, &Client::bufferRemoved);
+    connect(bufferSyncer(), &BufferSyncer::bufferRenamed, this, &Client::bufferRenamed);
+    connect(bufferSyncer(), &BufferSyncer::buffersPermanentlyMerged, this, &Client::buffersPermanentlyMerged);
+    connect(bufferSyncer(), &BufferSyncer::buffersPermanentlyMerged, _messageModel, &MessageModel::buffersPermanentlyMerged);
+    connect(bufferSyncer(), &BufferSyncer::bufferMarkedAsRead, this, &Client::bufferMarkedAsRead);
+    connect(bufferSyncer(), &BufferSyncer::bufferActivityChanged, _networkModel, &NetworkModel::bufferActivityChanged);
+    connect(bufferSyncer(), &BufferSyncer::highlightCountChanged, _networkModel, &NetworkModel::highlightCountChanged);
+    connect(networkModel(), &NetworkModel::requestSetLastSeenMsg, bufferSyncer(), &BufferSyncer::requestSetLastSeenMsg);
 
-    SignalProxy *p = signalProxy();
+    SignalProxy* p = signalProxy();
     p->synchronize(bufferSyncer());
 
     // create a new BufferViewManager
     Q_ASSERT(!_bufferViewManager);
     _bufferViewManager = new ClientBufferViewManager(p, this);
-    connect(_bufferViewManager, SIGNAL(initDone()), _bufferViewOverlay, SLOT(restore()));
+    connect(_bufferViewManager, &SyncableObject::initDone, _bufferViewOverlay, &BufferViewOverlay::restore);
 
     // create AliasManager
     Q_ASSERT(!_aliasManager);
     _aliasManager = new ClientAliasManager(this);
-    connect(aliasManager(), SIGNAL(initDone()), SLOT(sendBufferedUserInput()));
+    connect(aliasManager(), &SyncableObject::initDone, this, &Client::sendBufferedUserInput);
     p->synchronize(aliasManager());
 
     // create NetworkConfig
@@ -412,35 +359,78 @@ void Client::setSyncedToCore()
     _ignoreListManager = new ClientIgnoreListManager(this);
     p->synchronize(ignoreListManager());
 
-    Q_ASSERT(!_transferManager);
-    _transferManager = new ClientTransferManager(this);
-    p->synchronize(transferManager());
+    // create Core-Side HighlightRuleManager
+    Q_ASSERT(!_highlightRuleManager);
+    _highlightRuleManager = new HighlightRuleManager(this);
+    p->synchronize(highlightRuleManager());
+    // Listen to network removed events
+    connect(this, &Client::networkRemoved, _highlightRuleManager, &HighlightRuleManager::networkRemoved);
+
+    /*  not ready yet
+        // create TransferManager and DccConfig if core supports them
+        Q_ASSERT(!_dccConfig);
+        Q_ASSERT(!_transferManager);
+        if (isCoreFeatureEnabled(Quassel::Feature::DccFileTransfer)) {
+            _dccConfig = new DccConfig(this);
+            p->synchronize(dccConfig());
+            _transferManager = new ClientTransferManager(this);
+            _transferModel->setManager(_transferManager);
+            p->synchronize(transferManager());
+        }
+    */
 
     // trigger backlog request once all active bufferviews are initialized
-    connect(bufferViewOverlay(), SIGNAL(initDone()), this, SLOT(requestInitialBacklog()));
+    connect(bufferViewOverlay(), &BufferViewOverlay::initDone, this, &Client::finishConnectionInitialization);
 
     _connected = true;
     emit connected();
     emit coreConnectionStateChanged(true);
 }
 
-
-void Client::requestInitialBacklog()
+void Client::finishConnectionInitialization()
 {
     // usually it _should_ take longer until the bufferViews are initialized, so that's what
     // triggers this slot. But we have to make sure that we know all buffers yet.
     // so we check the BufferSyncer and in case it wasn't initialized we wait for that instead
     if (!bufferSyncer()->isInitialized()) {
-        disconnect(bufferViewOverlay(), SIGNAL(initDone()), this, SLOT(requestInitialBacklog()));
-        connect(bufferSyncer(), SIGNAL(initDone()), this, SLOT(requestInitialBacklog()));
+        disconnect(bufferViewOverlay(), &BufferViewOverlay::initDone, this, &Client::finishConnectionInitialization);
+        connect(bufferSyncer(), &SyncableObject::initDone, this, &Client::finishConnectionInitialization);
         return;
     }
-    disconnect(bufferViewOverlay(), SIGNAL(initDone()), this, SLOT(requestInitialBacklog()));
-    disconnect(bufferSyncer(), SIGNAL(initDone()), this, SLOT(requestInitialBacklog()));
+    disconnect(bufferViewOverlay(), &BufferViewOverlay::initDone, this, &Client::finishConnectionInitialization);
+    disconnect(bufferSyncer(), &SyncableObject::initDone, this, &Client::finishConnectionInitialization);
 
+    requestInitialBacklog();
+    if (isCoreFeatureEnabled(Quassel::Feature::BufferActivitySync)) {
+        bufferSyncer()->markActivitiesChanged();
+        bufferSyncer()->markHighlightCountsChanged();
+    }
+}
+
+void Client::requestInitialBacklog()
+{
     _backlogManager->requestInitialBacklog();
 }
 
+void Client::requestLegacyCoreInfo()
+{
+    // On older cores, the CoreInfo object was only synchronized on demand.  Synchronize now if
+    // needed.
+    if (isConnected() && !isCoreFeatureEnabled(Quassel::Feature::SyncedCoreInfo)) {
+        // Delete the existing core info object (it will always exist as client is single-threaded)
+        _coreInfo->deleteLater();
+        // No need to set to null when creating new one immediately after
+
+        // Create a fresh, unsynchronized CoreInfo object, emulating legacy behavior of CoreInfo not
+        // persisting
+        _coreInfo = new CoreInfo(this);
+        // Synchronize the new object
+        signalProxy()->synchronize(_coreInfo);
+
+        // Let others know signal handlers have been reset
+        emit coreInfoResynchronized();
+    }
+}
 
 void Client::disconnectFromCore()
 {
@@ -450,11 +440,9 @@ void Client::disconnectFromCore()
     coreConnection()->disconnectFromCore();
 }
 
-
 void Client::setDisconnectedFromCore()
 {
     _connected = false;
-    _coreFeatures = 0;
 
     emit disconnected();
     emit coreConnectionStateChanged(false);
@@ -466,29 +454,42 @@ void Client::setDisconnectedFromCore()
 
     if (_bufferSyncer) {
         _bufferSyncer->deleteLater();
-        _bufferSyncer = 0;
+        _bufferSyncer = nullptr;
     }
 
+    _coreInfo->reset();
+
     if (_bufferViewManager) {
         _bufferViewManager->deleteLater();
-        _bufferViewManager = 0;
+        _bufferViewManager = nullptr;
     }
 
     _bufferViewOverlay->reset();
 
     if (_aliasManager) {
         _aliasManager->deleteLater();
-        _aliasManager = 0;
+        _aliasManager = nullptr;
     }
 
     if (_ignoreListManager) {
         _ignoreListManager->deleteLater();
-        _ignoreListManager = 0;
+        _ignoreListManager = nullptr;
+    }
+
+    if (_highlightRuleManager) {
+        _highlightRuleManager->deleteLater();
+        _highlightRuleManager = nullptr;
     }
 
     if (_transferManager) {
+        _transferModel->setManager(nullptr);
         _transferManager->deleteLater();
-        _transferManager = 0;
+        _transferManager = nullptr;
+    }
+
+    if (_dccConfig) {
+        _dccConfig->deleteLater();
+        _dccConfig = nullptr;
     }
 
     // we probably don't want to save pending input for reconnect
@@ -497,20 +498,20 @@ void Client::setDisconnectedFromCore()
     _messageModel->clear();
     _networkModel->clear();
 
-    QHash<NetworkId, Network *>::iterator netIter = _networks.begin();
+    QHash<NetworkId, Network*>::iterator netIter = _networks.begin();
     while (netIter != _networks.end()) {
-        Network *net = netIter.value();
+        Network* net = netIter.value();
         emit networkRemoved(net->networkId());
-        disconnect(net, SIGNAL(destroyed()), this, 0);
+        disconnect(net, &Network::destroyed, this, nullptr);
         netIter = _networks.erase(netIter);
         net->deleteLater();
     }
     Q_ASSERT(_networks.isEmpty());
 
-    QHash<IdentityId, Identity *>::iterator idIter = _identities.begin();
+    QHash<IdentityId, Identity*>::iterator idIter = _identities.begin();
     while (idIter != _identities.end()) {
         emit identityRemoved(idIter.key());
-        Identity *id = idIter.value();
+        Identity* id = idIter.value();
         idIter = _identities.erase(idIter);
         id->deleteLater();
     }
@@ -518,17 +519,16 @@ void Client::setDisconnectedFromCore()
 
     if (_networkConfig) {
         _networkConfig->deleteLater();
-        _networkConfig = 0;
+        _networkConfig = nullptr;
     }
 }
 
-
 /*** ***/
 
 void Client::networkDestroyed()
 {
-    Network *net = static_cast<Network *>(sender());
-    QHash<NetworkId, Network *>::iterator netIter = _networks.begin();
+    auto* net = static_cast<Network*>(sender());
+    QHash<NetworkId, Network*>::iterator netIter = _networks.begin();
     while (netIter != _networks.end()) {
         if (*netIter == net) {
             netIter = _networks.erase(netIter);
@@ -540,58 +540,51 @@ void Client::networkDestroyed()
     }
 }
 
-
 // Hmm... we never used this...
 void Client::recvStatusMsg(QString /*net*/, QString /*msg*/)
 {
-    //recvMessage(net, Message::server("", QString("[STATUS] %1").arg(msg)));
+    // recvMessage(net, Message::server("", QString("[STATUS] %1").arg(msg)));
 }
 
-
-void Client::recvMessage(const Message &msg)
+void Client::recvMessage(const Message& msg)
 {
     Message msg_ = msg;
     messageProcessor()->process(msg_);
 }
 
-
-void Client::setBufferLastSeenMsg(BufferId id, const MsgId &msgId)
+void Client::setBufferLastSeenMsg(BufferId id, const MsgId& msgId)
 {
     if (bufferSyncer())
         bufferSyncer()->requestSetLastSeenMsg(id, msgId);
 }
 
-
-void Client::setMarkerLine(BufferId id, const MsgId &msgId)
+void Client::setMarkerLine(BufferId id, const MsgId& msgId)
 {
     if (bufferSyncer())
         bufferSyncer()->requestSetMarkerLine(id, msgId);
 }
 
-
 MsgId Client::markerLine(BufferId id)
 {
     if (id.isValid() && networkModel())
         return networkModel()->markerLineMsgId(id);
-    return MsgId();
+    return {};
 }
 
-
 void Client::removeBuffer(BufferId id)
 {
-    if (!bufferSyncer()) return;
+    if (!bufferSyncer())
+        return;
     bufferSyncer()->requestRemoveBuffer(id);
 }
 
-
-void Client::renameBuffer(BufferId bufferId, const QString &newName)
+void Client::renameBuffer(BufferId bufferId, const QString& newName)
 {
     if (!bufferSyncer())
         return;
     bufferSyncer()->requestRenameBuffer(bufferId, newName);
 }
 
-
 void Client::mergeBuffersPermanently(BufferId bufferId1, BufferId bufferId2)
 {
     if (!bufferSyncer())
@@ -599,7 +592,6 @@ void Client::mergeBuffersPermanently(BufferId bufferId1, BufferId bufferId2)
     bufferSyncer()->requestMergeBuffersPermanently(bufferId1, bufferId2);
 }
 
-
 void Client::purgeKnownBufferIds()
 {
     if (!bufferSyncer())
@@ -607,7 +599,6 @@ void Client::purgeKnownBufferIds()
     bufferSyncer()->requestPurgeBufferIds();
 }
 
-
 void Client::bufferRemoved(BufferId bufferId)
 {
     // select a sane buffer (status buffer)
@@ -627,8 +618,7 @@ void Client::bufferRemoved(BufferId bufferId)
     networkModel()->removeBuffer(bufferId);
 }
 
-
-void Client::bufferRenamed(BufferId bufferId, const QString &newName)
+void Client::bufferRenamed(BufferId bufferId, const QString& newName)
 {
     QModelIndex bufferIndex = networkModel()->bufferIndex(bufferId);
     if (bufferIndex.isValid()) {
@@ -636,7 +626,6 @@ void Client::bufferRenamed(BufferId bufferId, const QString &newName)
     }
 }
 
-
 void Client::buffersPermanentlyMerged(BufferId bufferId1, BufferId bufferId2)
 {
     QModelIndex idx = networkModel()->bufferIndex(bufferId1);
@@ -644,21 +633,29 @@ void Client::buffersPermanentlyMerged(BufferId bufferId1, BufferId bufferId2)
     networkModel()->removeBuffer(bufferId2);
 }
 
-
 void Client::markBufferAsRead(BufferId id)
 {
     if (bufferSyncer() && id.isValid())
         bufferSyncer()->requestMarkBufferAsRead(id);
 }
 
+void Client::refreshLegacyCoreInfo()
+{
+    instance()->requestLegacyCoreInfo();
+}
 
-void Client::changePassword(const QString &oldPassword, const QString &newPassword) {
+void Client::changePassword(const QString& oldPassword, const QString& newPassword)
+{
     CoreAccount account = currentCoreAccount();
     account.setPassword(newPassword);
     coreAccountModel()->createOrUpdateAccount(account);
     emit instance()->requestPasswordChange(nullptr, account.user(), oldPassword, newPassword);
 }
 
+void Client::kickClient(int peerId)
+{
+    emit instance()->requestKickClient(peerId);
+}
 
 void Client::corePasswordChanged(PeerPtr, bool success)
 {
@@ -666,48 +663,3 @@ void Client::corePasswordChanged(PeerPtr, bool success)
         coreAccountModel()->save();
     emit passwordChanged(success);
 }
-
-
-#if QT_VERSION < 0x050000
-void Client::logMessage(QtMsgType type, const char *msg)
-{
-    fprintf(stderr, "%s\n", msg);
-    fflush(stderr);
-    if (type == QtFatalMsg) {
-        Quassel::logFatalMessage(msg);
-    }
-    else {
-        QString msgString = QString("%1\n").arg(msg);
-
-        //Check to see if there is an instance around, else we risk recursions
-        //when calling instance() and creating new ones.
-        if (!instanceExists())
-            return;
-
-        instance()->_debugLog << msgString;
-        emit instance()->logUpdated(msgString);
-    }
-}
-#else
-void Client::logMessage(QtMsgType type, const QMessageLogContext &context, const QString &msg)
-{
-    Q_UNUSED(context);
-
-    fprintf(stderr, "%s\n", msg.toLocal8Bit().constData());
-    fflush(stderr);
-    if (type == QtFatalMsg) {
-        Quassel::logFatalMessage(msg.toLocal8Bit().constData());
-    }
-    else {
-        QString msgString = QString("%1\n").arg(msg);
-
-        //Check to see if there is an instance around, else we risk recursions
-        //when calling instance() and creating new ones.
-        if (!instanceExists())
-            return;
-
-        instance()->_debugLog << msgString;
-        emit instance()->logUpdated(msgString);
-    }
-}
-#endif
