@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2015 by the Quassel Project                        *
+ *   Copyright (C) 2005-2022 by the Quassel Project                        *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,18 +18,18 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
-#ifndef CLIENTBUFFERVIEWCONFIG_H
-#define CLIENTBUFFERVIEWCONFIG_H
+#pragma once
+
+#include "client-export.h"
 
 #include "bufferviewconfig.h"
 
-class ClientBufferViewConfig : public BufferViewConfig
+class CLIENT_EXPORT ClientBufferViewConfig : public BufferViewConfig
 {
-    SYNCABLE_OBJECT
-        Q_OBJECT
+    Q_OBJECT
 
 public:
-    ClientBufferViewConfig(int bufferViewId, QObject *parent = 0);
+    ClientBufferViewConfig(int bufferViewId, QObject* parent = nullptr);
 
     inline bool isLocked() { return _locked || sortAlphabetically(); }
     inline void setLocked(bool locked) { _locked = locked; }
@@ -37,11 +37,8 @@ public:
     inline void unlock() { setLocked(false); };
 
 private slots:
-    void ensureDecoration(); // remove this in next release
+    void ensureDecoration();  // remove this in next release
 
 private:
     bool _locked;
 };
-
-
-#endif //CLIENTBUFFERVIEWCONFIG_H
