@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2015 by the Quassel Project                        *
+ *   Copyright (C) 2005-2022 by the Quassel Project                        *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,13 +20,13 @@
 
 #include "ircevent.h"
 
-Event *IrcEvent::create(EventManager::EventType type, QVariantMap &map, Network *network)
+Event* IrcEvent::create(EventManager::EventType type, QVariantMap& map, Network* network)
 {
     if ((type & ~EventManager::IrcEventNumericMask) == EventManager::IrcEventNumeric)
         return new IrcEventNumeric(type, map, network);
 
     if ((type & EventManager::EventGroupMask) != EventManager::IrcEvent)
-        return 0;
+        return nullptr;
 
     switch (type) {
     case EventManager::IrcEventRawPrivmsg:
@@ -38,47 +38,41 @@ Event *IrcEvent::create(EventManager::EventType type, QVariantMap &map, Network
     }
 }
 
-
-IrcEvent::IrcEvent(EventManager::EventType type, QVariantMap &map, Network *network)
+IrcEvent::IrcEvent(EventManager::EventType type, QVariantMap& map, Network* network)
     : NetworkEvent(type, map, network)
 {
     _prefix = map.take("prefix").toString();
     _params = map.take("params").toStringList();
 }
 
-
-void IrcEvent::toVariantMap(QVariantMap &map) const
+void IrcEvent::toVariantMap(QVariantMap& map) const
 {
     NetworkEvent::toVariantMap(map);
     map["prefix"] = prefix();
     map["params"] = params();
 }
 
-
-IrcEventNumeric::IrcEventNumeric(EventManager::EventType type, QVariantMap &map, Network *network)
+IrcEventNumeric::IrcEventNumeric(EventManager::EventType type, QVariantMap& map, Network* network)
     : IrcEvent(type, map, network)
 {
     _number = map.take("number").toUInt();
     _target = map.take("target").toString();
 }
 
-
-void IrcEventNumeric::toVariantMap(QVariantMap &map) const
+void IrcEventNumeric::toVariantMap(QVariantMap& map) const
 {
     IrcEvent::toVariantMap(map);
     map["number"] = number();
     map["target"] = target();
 }
 
-
-IrcEventRawMessage::IrcEventRawMessage(EventManager::EventType type, QVariantMap &map, Network *network)
+IrcEventRawMessage::IrcEventRawMessage(EventManager::EventType type, QVariantMap& map, Network* network)
     : IrcEvent(type, map, network)
 {
     _rawMessage = map.take("rawMessage").toByteArray();
 }
 
-
-void IrcEventRawMessage::toVariantMap(QVariantMap &map) const
+void IrcEventRawMessage::toVariantMap(QVariantMap& map) const
 {
     IrcEvent::toVariantMap(map);
     map["rawMessage"] = rawMessage();
