@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2015 by the Quassel Project                        *
+ *   Copyright (C) 2005-2022 by the Quassel Project                        *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,31 +20,26 @@
 
 #include "verticaldock.h"
 
-#include <qdrawutil.h>
+#include <QDebug>
 #include <QLayout>
 #include <QPainter>
+#include <qdrawutil.h>
 
-#include <QDebug>
-
-VerticalDockTitle::VerticalDockTitle(QDockWidget *parent)
+VerticalDockTitle::VerticalDockTitle(QDockWidget* parent)
     : QWidget(parent)
-{
-}
-
+{}
 
 QSize VerticalDockTitle::sizeHint() const
 {
-    return QSize(8, 15);
+    return {8, 15};
 }
 
-
 QSize VerticalDockTitle::minimumSizeHint() const
 {
-    return QSize(8, 10);
+    return {8, 10};
 }
 
-
-void VerticalDockTitle::paintEvent(QPaintEvent *event)
+void VerticalDockTitle::paintEvent(QPaintEvent* event)
 {
     Q_UNUSED(event);
 
@@ -52,36 +47,33 @@ void VerticalDockTitle::paintEvent(QPaintEvent *event)
 
     if (rect().isValid() && rect().height() > minimumSizeHint().height()) {
         for (int i = 0; i < 2; i++) {
-            QPoint topLeft = rect().topLeft() + QPoint(3 + i*2, 2);
-            QPoint bottomRight = rect().topLeft() + QPoint(3 + i*2, rect().height() - 2);
+            QPoint topLeft = rect().topLeft() + QPoint(3 + i * 2, 2);
+            QPoint bottomRight = rect().topLeft() + QPoint(3 + i * 2, rect().height() - 2);
             qDrawShadeLine(&painter, topLeft, bottomRight, palette());
         }
     }
 }
 
-
 // ==============================
 //  Vertical Dock
 // ==============================
-VerticalDock::VerticalDock(const QString &title, QWidget *parent, Qt::WindowFlags flags)
+VerticalDock::VerticalDock(const QString& title, QWidget* parent, Qt::WindowFlags flags)
     : QDockWidget(title, parent, flags)
 {
     setDefaultTitleWidget();
 }
 
-
-VerticalDock::VerticalDock(QWidget *parent, Qt::WindowFlags flags)
+VerticalDock::VerticalDock(QWidget* parent, Qt::WindowFlags flags)
     : QDockWidget(parent, flags)
 {
     setDefaultTitleWidget();
     setContentsMargins(0, 0, 0, 0);
 }
 
-
 void VerticalDock::setDefaultTitleWidget()
 {
-    QWidget *oldDockTitle = titleBarWidget();
-    QWidget *newDockTitle = new VerticalDockTitle(this);
+    QWidget* oldDockTitle = titleBarWidget();
+    QWidget* newDockTitle = new VerticalDockTitle(this);
 
     setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
     setFeatures(features() | QDockWidget::DockWidgetVerticalTitleBar);
@@ -91,11 +83,10 @@ void VerticalDock::setDefaultTitleWidget()
         oldDockTitle->deleteLater();
 }
 
-
 void VerticalDock::showTitle(bool show)
 {
-    QWidget *oldDockTitle = titleBarWidget();
-    QWidget *newDockTitle = 0;
+    QWidget* oldDockTitle = titleBarWidget();
+    QWidget* newDockTitle = nullptr;
 
     if (show)
         newDockTitle = new VerticalDockTitle(this);
